@@ -15,8 +15,13 @@ Coxph <- function(formula, data, subset, weights, offset, cluster, na.action = n
                 negative = FALSE, ...)
     if (!inherits(ret, "mlt")) return(ret)
     ret$call <- match.call(expand.dots = TRUE)
-    ret$tram <- paste(ifelse(is.null(td$terms$s), "", "(Stratified)"), 
-                      "Parametric Linear Cox Regression Model")
+    if (!is.null(td$mt$z)) {
+        ret$tram <- paste(ifelse(is.null(td$mt$s), "", "(Stratified)"),
+                          "Shift-Scale Transformation Model")
+    } else {
+        ret$tram <- paste(ifelse(is.null(td$mt$s), "", "(Stratified)"), 
+                          "Parametric Linear Cox Regression Model")
+    }
     class(ret) <- c("Coxph", class(ret))
     ret
 }
@@ -77,8 +82,13 @@ Aareg <- function(formula, data, subset, weights, offset, cluster, na.action = n
 
     if (!inherits(ret, "mlt")) return(ret)
     ret$call <- match.call(expand.dots = TRUE)
-    ret$tram <- paste(ifelse(is.null(td$terms$s), "", "(Stratified)"), 
-                      "Parametric Linear Aalen Regression Model")
+    if (!is.null(td$mt$z)) {
+        ret$tram <- paste(ifelse(is.null(td$mt$s), "", "(Stratified)"),
+                          "Shift-Scale Transformation Model")
+    } else {
+        ret$tram <- paste(ifelse(is.null(td$mt$s), "", "(Stratified)"), 
+                          "Parametric Linear Aalen Regression Model")
+    }
     class(ret) <- c("Aareg", class(ret))
     ret
 }
@@ -169,8 +179,13 @@ Survreg <- function(formula, data, subset, weights, offset, cluster, na.action =
                sep = "", collapse = " ")
     }
 
-    ret$tram <- paste(ifelse(is.null(td$terms$s), "", "(Stratified)"),
-                      .simpleCap(dist), "Linear Regression Model")
+    if (!is.null(td$mt$z)) {
+        ret$tram <- paste(ifelse(is.null(td$mt$s), "", "(Stratified)"),
+                          "Shift-Scale Transformation Model")
+    } else {
+        ret$tram <- paste(ifelse(is.null(td$mt$s), "", "(Stratified)"),
+                          .simpleCap(dist), "Linear Regression Model")
+    }
     class(ret) <- c("Survreg", class(ret))
     ret
 }
@@ -191,8 +206,13 @@ Colr <- function(formula, data, subset, weights, offset, cluster, na.action = na
                 distribution = "Logistic", negative = FALSE, ...)
     if (!inherits(ret, "mlt")) return(ret)
     ret$call <- match.call(expand.dots = TRUE)
-    ret$tram <- paste(ifelse(is.null(td$terms$s), "", "(Stratified)"),
-                      "Continuous Outcome Logistic Regression")
+    if (!is.null(td$mt$z)) {
+        ret$tram <- paste(ifelse(is.null(td$mt$s), "", "(Stratified)"),
+                          "Shift-Scale Transformation Model")
+    } else {
+        ret$tram <- paste(ifelse(is.null(td$mt$s), "", "(Stratified)"),
+                          "Continuous Outcome Logistic Regression")
+    }
     class(ret) <- c("Colr", class(ret))
     ret
 }
@@ -220,13 +240,18 @@ Polr <- function(formula, data, subset, weights, offset, cluster, na.action = na
     ret <- tram(td, transformation = "discrete", distribution = distribution, negative = TRUE, ...)
     if (!inherits(ret, "mlt")) return(ret)
     ret$call <- match.call(expand.dots = TRUE)
-    if (method != "probit") {
-        ret$tram <- paste(ifelse(is.null(td$terms$s), "", "(Stratified)"),
-                          "Proportional", name[method], "Regression Model")
+    if (!is.null(td$mt$z)) {
+        ret$tram <- paste(ifelse(is.null(td$mt$s), "", "(Stratified)"),
+                          "Shift-Scale Transformation Model")
     } else {
-        ret$tram <- paste(ifelse(is.null(td$terms$s), "", "(Stratified)"),
-                          "Ordered Probit Regression Model")
-    }                   
+        if (method != "probit") {
+            ret$tram <- paste(ifelse(is.null(td$mt$s), "", "(Stratified)"),
+                              "Proportional", name[method], "Regression Model")
+        } else {
+            ret$tram <- paste(ifelse(is.null(td$mt$s), "", "(Stratified)"),
+                              "Ordered Probit Regression Model")
+        }                   
+    }
     class(ret) <- c("Polr", class(ret))
     ret
 }
@@ -247,8 +272,13 @@ Lm <- function(formula, data, subset, weights, offset, cluster, na.action = na.o
                 negative = TRUE, ...)
     if (!inherits(ret, "mlt")) return(ret)
     ret$call <- match.call(expand.dots = TRUE)
-    ret$tram <- paste(ifelse(is.null(td$terms$s), "", "(Stratified)"),
-                      "Normal Linear Regression Model")
+    if (!is.null(td$mt$z)) {
+        ret$tram <- paste(ifelse(is.null(td$mt$s), "", "(Stratified)"),
+                          "Shift-Scale Normal Regression Model")
+    } else {
+        ret$tram <- paste(ifelse(is.null(td$mt$s), "", "(Stratified)"),
+                          "Normal Linear Regression Model")
+    }
     class(ret) <- c("Lm", class(ret))
     ret
 }
@@ -269,8 +299,13 @@ BoxCox <- function(formula, data, subset, weights, offset, cluster, na.action =
                 negative = TRUE, ...)
     if (!inherits(ret, "mlt")) return(ret)
     ret$call <- match.call(expand.dots = TRUE)
-    ret$tram <- paste(ifelse(is.null(td$terms$s), "", "(Stratified)"),
-                      "Non-normal (Box-Cox-Type) Linear Regression Model")
+    if (!is.null(td$mt$z)) {
+        ret$tram <- paste(ifelse(is.null(td$mt$s), "", "(Stratified)"),
+                          "Shift-Scale Transformation Model")
+    } else {
+        ret$tram <- paste(ifelse(is.null(td$mt$s), "", "(Stratified)"),
+                          "Non-normal (Box-Cox-Type) Linear Regression Model")
+    }
     class(ret) <- c("BoxCox", class(ret))
     ret
 }
@@ -291,8 +326,13 @@ Lehmann <- function(formula, data, subset, weights, offset, cluster, na.action =
                 negative = TRUE, ...)
     if (!inherits(ret, "mlt")) return(ret)
     ret$call <- match.call(expand.dots = TRUE)
-    ret$tram <- paste(ifelse(is.null(td$terms$s), "", "(Stratified)"),
-                      "Proportional Reverse Time Hazards Linear Regression Model")
+    if (!is.null(td$mt$z)) {
+        ret$tram <- paste(ifelse(is.null(td$mt$s), "", "(Stratified)"),
+                          "Shift-Scale Transformation Model")
+    } else {
+        ret$tram <- paste(ifelse(is.null(td$mt$s), "", "(Stratified)"),
+                          "Proportional Reverse Time Hazards Linear Regression Model")
+    }
     class(ret) <- c("Lehmann", class(ret))
     ret
 }
